package tamanegisoul.ar.action;

import tamanegisoul.ar.Utils;
import tamanegisoul.ar.db.DBHelper;
import tamanegisoul.ar.db.DataChangeListener;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.IBinder;

/**
 * GPS位置情報をポーリングする。
 * @author TamanegiSoul
 */
public class LocationPollingService extends Service implements DataChangeListener {

	private static class MyLocationListener implements LocationListener {
		private Context mContext;
		private NotificationManager mNotificationManager;
		private SQLiteDatabase mDatabase;
		private Cursor mLocationCursor;
		private int mNotifyDistance;

		public MyLocationListener(Context context, NotificationManager notificationManager, SQLiteDatabase db, Cursor locationCursor,
				int notifyDistance) {
			super();
			mContext = context;
			mNotificationManager = notificationManager;
			mDatabase = db;
			mLocationCursor = locationCursor;
			mNotifyDistance = notifyDistance;
		}

		public void onStatusChanged(String arg0, int arg1, Bundle arg2) {
		}

		public void onProviderEnabled(String arg0) {
		}

		public void onProviderDisabled(String arg0) {
		}

		public void onLocationChanged(Location loc) {
			// Destroyした後にも呼ばれるので対処
			if (!mLocationCursor.isClosed()) {
				for (mLocationCursor.moveToFirst(); !mLocationCursor.isAfterLast(); mLocationCursor.moveToNext()) {
					// 現在地から登録地点までの距離を算出
					int lat = mLocationCursor.getInt(2);
					int lon = mLocationCursor.getInt(3);
					float[] distance = new float[3];
					Location.distanceBetween(lat / 1E6, lon / 1E6, loc.getLatitude(), loc.getLongitude(), distance);
					// 登録地点が近づいたら通知
					if (distance[0] < mNotifyDistance) {
						Cursor todoCursor = mDatabase.query(DBHelper.TABLE_TODO, new String[] { DBHelper.TABLE_TODO_COLUMN_NAME },
								DBHelper.TABLE_TODO_COLUMN_LOCATION_ID + "=? and " + DBHelper.TABLE_TODO_COLUMN_NOTIFIED + "=0",
								new String[] { mLocationCursor.getString(0) }, null, null, null);
						if (todoCursor.getCount() != 0) {
							String title = String.format("%sまであと%dメートル", mLocationCursor.getString(1), Math.round(distance[0]));
							StringBuilder message = new StringBuilder();
							for (todoCursor.moveToFirst(); !todoCursor.isAfterLast(); todoCursor.moveToNext()) {
								message.append(todoCursor.getString(0)).append(" ");
							}
							Notification notification = new Notification(android.R.drawable.ic_dialog_info, title, System.currentTimeMillis());
							PendingIntent contentIntent = PendingIntent.getActivity(mContext, 0, new Intent(mContext, MainActivity.class),
									Intent.FLAG_ACTIVITY_NEW_TASK);
							notification.setLatestEventInfo(mContext, "AtReminder", message, contentIntent);
							mNotificationManager.notify(mLocationCursor.getInt(0), notification);
							// 通知済みとしてDBを更新
							ContentValues values = new ContentValues();
							values.put(DBHelper.TABLE_TODO_COLUMN_NOTIFIED, 1);
							mDatabase.update(DBHelper.TABLE_TODO, values, DBHelper.TABLE_TODO_COLUMN_LOCATION_ID + "=?", new String[] { mLocationCursor
									.getString(0) });
							DBHelper.notifyDataChanged(DBHelper.TABLE_TODO);
							// ベルを鳴らす
							Uri uri = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_RINGTONE);
							Ringtone ringtone = RingtoneManager.getRingtone(mContext, uri);
							ringtone.play();
						}
						todoCursor.close();
					}
				}
			}
		}
	}

	private LocationManager mLocationManager;
	private MyLocationListener mLocationListener;
	private Cursor mNotifyDistanceSettingCursor;
	private Cursor mTimeSettingCursor;
	private Cursor mDistanceSettingCursor;
	private Cursor mLocationCursor;
	private SQLiteDatabase mDatabase;

	@Override
	public void onCreate() {
		Utils.logDebug(this.getClass(), "onCreate()");
		super.onCreate();
		
		mDatabase = DBHelper.getDatabase(this);
		DBHelper.registerDataChangeListener(this);

		mNotifyDistanceSettingCursor = mDatabase.query(DBHelper.TABLE_SETTINGS, null, DBHelper.TABLE_SETTINGS_COLUMN_NAME + "=?",
				new String[] { DBHelper.TABLE_SETTINGS_COLUMN_NAME_NOTIFY_DISTANCE}, null, null, null);
		mTimeSettingCursor = mDatabase.query(DBHelper.TABLE_SETTINGS, null, DBHelper.TABLE_SETTINGS_COLUMN_NAME + "=?",
				new String[] { DBHelper.TABLE_SETTINGS_COLUMN_NAME_POLLING_TIME }, null, null, null);
		mDistanceSettingCursor = mDatabase.query(DBHelper.TABLE_SETTINGS, null, DBHelper.TABLE_SETTINGS_COLUMN_NAME + "=?",
				new String[] { DBHelper.TABLE_SETTINGS_COLUMN_NAME_POLLING_DISTANCE }, null, null, null);
		mLocationCursor = mDatabase.query(DBHelper.TABLE_LOCATION, null, null, null, null, null, null);

		restartPolling();
	}

	@Override
	public void onDestroy() {
		Utils.logDebug(this.getClass(), "onDestroy()");
		super.onDestroy();

		mNotifyDistanceSettingCursor.close();
		mTimeSettingCursor.close();
		mDistanceSettingCursor.close();
		mLocationCursor.close();
		
		DBHelper.unregisterDataChangeListener(this);
		mDatabase.close();

		mLocationManager.removeUpdates(mLocationListener);
	}

	public void onDataChanged(String tableName) {
		if (tableName.equals(DBHelper.TABLE_LOCATION))
			mLocationCursor.requery();
		if (tableName.equals(DBHelper.TABLE_SETTINGS))
			restartPolling();
	}

	@Override
	public IBinder onBind(Intent arg0) {
		return null;
	}

	/**
	 * GPSポーリングを開始する
	 */
	private void restartPolling() {
		if (mLocationListener != null)
			mLocationManager.removeUpdates(mLocationListener);

		// 設定値取得
		mNotifyDistanceSettingCursor.requery();
		mNotifyDistanceSettingCursor.moveToFirst();
		int notifyDistance = mNotifyDistanceSettingCursor.getInt(2);
		mTimeSettingCursor.requery();
		mTimeSettingCursor.moveToFirst();
		int time = mTimeSettingCursor.getInt(2);
		mDistanceSettingCursor.requery();
		mDistanceSettingCursor.moveToFirst();
		int distance = mDistanceSettingCursor.getInt(2);

		// ポーリング開始
		NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
		mLocationListener = new MyLocationListener(this, notificationManager, mDatabase, mLocationCursor, notifyDistance);
		mLocationManager = (LocationManager) getApplicationContext().getSystemService(Context.LOCATION_SERVICE);
		mLocationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, time * 60 * 1000, distance, mLocationListener);
	}
}
